"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const js_utils_1 = require("@ptolemy2002/js-utils");
(0, js_utils_1.loadExtension)("hasProperty", js_utils_1.ext_hasProperty, Object);
// The hook result is both iterable and allows direct access to properties.
class HookResult {
    constructor(data, order = []) {
        this._data = new Map();
        if (typeof Proxy == "undefined")
            throw new Error("Proxy is not supported in this environment.");
        // Set the ordered keys first
        order.forEach(key => {
            this._data.set(key, data[key]);
        });
        // Set any remaining keys in the order they appear in the object
        Object.keys(data).forEach(key => {
            if (!order.includes(key)) {
                this._data.set(key, data[key]);
            }
        });
        return new Proxy(this, {
            // If the property is not found on the object, get it from the data map
            get: function (target, prop) {
                if (target.hasProperty(prop)) {
                    return Reflect.get(target, prop);
                }
                else {
                    return target._data.get(prop);
                }
            },
            // If the property is not found on the object, set it in the data map
            set: function (target, prop, value) {
                if (target.hasProperty(prop)) {
                    Reflect.set(target, prop, value);
                }
                else {
                    target._data.set(prop, value);
                }
                return value;
            }
        });
    }
    // This is a generator function: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Statements/function*
    *[Symbol.iterator]() {
        yield* this._data.values();
    }
}
exports.default = HookResult;
